// Exemple DirectDraw - Chapitre 2
// DATE:	16/06/99
// AUTEUR:	Shaun Dor
// NOTE:	Ce programme dmontre la base de DirectDraw:
//			 - Cration d'une surface primaire
//			 - Cration d'une surface secondaire
//			 - Affiche des pixels dans la surface secondaire
//			 - Flip les surfaces

// MACROS ///////////////////////////////////////////////////////////////

// Rduit la taille des headers Windows
#define WIN32_LEAN_AND_MEAN  
// Macro qui retourne vrai si une touche est enfonce
#define Touche(VkCode) (GetAsyncKeyState(VkCode) & 0x8000 ? 1 : 0)
// Taille X de l'cran
#define ECRAN_X 640
// Taille Y de l'cran
#define ECRAN_Y 480


// INCLUDES /////////////////////////////////////////////////////////////

#include <windows.h>	// Include standard de tout programme Win32
#include <windowsx.h>	// Fonctionnalits supplmentaires de Win32
#include <ddraw.h>		// Header de DirectDraw


// GLOBALES /////////////////////////////////////////////////////////////

LPDIRECTDRAW		DirectDraw			= NULL;		// Objet DirectDraw
LPDIRECTDRAWSURFACE SurfacePrimaire		= NULL;		// Surface primaire
LPDIRECTDRAWSURFACE SurfaceSecondaire	= NULL;		// L'ecran virtuel de la surface primaire
DDSURFACEDESC		SurfaceDesc;					// Description de la surface
DDSCAPS				SurfaceCap;						// Capacits de la surface
HINSTANCE			InstancePrincipale	= NULL;		// Instance de l'application
HWND				HandlePrincipale	= NULL;		// Handle sur la fentre principale
UCHAR				*virtuel			= NULL;		// Ptr sur cran virtuel

// DCLARATIONS /////////////////////////////////////////////////////////

int  InitDirectDraw();		// Initialisation de DirectDraw
void CloseDirectDraw();		// Libre la mmoire des objets DirectDraw
void DessinePixel();		// Dessine des pixels sur l'cran secondaire
int  InitFenetre();			// Initialise la fentre

// FONCTIONS ////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Gestionnaire d'vnements de la fentre
LRESULT CALLBACK WindowProc(HWND hWnd,     // Handle sur la fentre
                            UINT Msg,      // Identificateur du message
                            WPARAM wParam, // Paramtre supplmentaire
                            LPARAM lParam) // Paramtre supplmentaire
{
	// Analyse le message
	switch (Msg)
	{	
		// La fentre va tre dtruite
		case WM_DESTROY:
		{
			// Envoie un message  la fentre pour quitter
			PostQuitMessage(0);
			return 0;
		}	break;
		
		default:
			break;
	}
	// Le reste des messages au gestionnaire par dfaut
	return DefWindowProc(hWnd, Msg, wParam, lParam);
}

////////////////////////////////////////////////////////////////////////////
// Initialise et instancie une fentre
int InitFenetre()
{
	WNDCLASS ClasseFenetre;	// Le nom de notre classe
	HWND hWnd = NULL;		// Handle sur la fentre

	// Style de la fentre
	ClasseFenetre.style = CS_DBLCLKS | CS_OWNDC | CS_HREDRAW | CS_VREDRAW;
	// Gestionnaire d'vnements de la fentre
	ClasseFenetre.lpfnWndProc = WindowProc;
	// Paramtres supplmentaires
	ClasseFenetre.cbClsExtra = 0;
	// Paramtres supplmentaires
	ClasseFenetre.cbWndExtra = 0;
	// Ptr sur instance de l'application
	ClasseFenetre.hInstance = InstancePrincipale;
	// Icne de la fentre
	ClasseFenetre.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	// Curseur de la fentre
	ClasseFenetre.hCursor = LoadCursor(NULL, IDC_ARROW);
	// Palette de couleur
	ClasseFenetre.hbrBackground = (struct HBRUSH__ *)GetStockObject(BLACK_BRUSH);
	// Ptr sur menu
	ClasseFenetre.lpszMenuName = NULL;
	// Ptr sur nom de la classe
	ClasseFenetre.lpszClassName = "Fenetre";
	
	// Enregistre la classe
	if (!RegisterClass(&ClasseFenetre))
	{
		MessageBox(NULL,
		           "Il y a eu une erreur dans l'enregistrement de la classe",
		           "Erreur",
		           MB_OK | MB_ICONERROR);
		return 0;
	};
	
	// Cration de la fentre
	hWnd = CreateWindow("Fenetre",              
	                    "Programmation Windows - Chapitre 2",          
	                    WS_POPUP | WS_VISIBLE, 
	                    CW_USEDEFAULT,         
	                    CW_USEDEFAULT,        
	                    CW_USEDEFAULT,         
	                    CW_USEDEFAULT,         
	                    NULL,                  
	                    NULL,                  
	                    InstancePrincipale,    
	                    NULL);   
	// Vrifie l'initialisation de la fentre
	if (!hWnd)
	{
		MessageBox(NULL,
		           "Il y a eu une erreur lors de la cration de la fentre",
		           "Erreur",
		           MB_OK | MB_ICONERROR);
		return 0;
	}
	// Sauvegarde l'handle de la fentre dans une globale
	HandlePrincipale = hWnd;
	return 1;
}

////////////////////////////////////////////////////////////////////////////
// Fonction principale
int WINAPI WinMain(HINSTANCE hInstance,     // Handle sur l'instance prsente
                   HINSTANCE hPrevInstance, // Handle sur l'instance prcdente
                   LPSTR lpCmbLine,         // Ptr sur les arguments
                   int nCmdShow)            // tat de la fentre
{
	MSG	Msg;
	BOOL Quitter = FALSE;			// Test pour savoir si on quitte l'app	
	InstancePrincipale = hInstance; // Sauvegarde de l'instance principale

	// Initialise et affiche la fentre principale
	if (!InitFenetre())
	{
		return 0;
	}

	// Initialise DirectDraw
	if (!InitDirectDraw())
	{
		// S'il y a une erreur, afficher un messagebox et quitter l'application
		MessageBox(HandlePrincipale,"Erreur lors de l'initialisation de DirectDraw",
						"Erreur DirectDraw",MB_OK | MB_ICONERROR);
		Quitter = TRUE;
	}

	// Tant qu'on n'a pas quitter
	while (Quitter == FALSE)
	{
		// Regarde s'il y a un message dans la msg queue
		if (PeekMessage(&Msg, NULL, 0, 0, PM_REMOVE))
		{
			// Vrifie si le message est un WM_QUIT
			if (Msg.message == WM_QUIT)
				Quitter = true;
			else
			{
				// Traduit le message
				TranslateMessage(&Msg);
				// Envoie le message au gestionnaire d'vnements
				DispatchMessage(&Msg);
			}
		}
		// Dessine les pixels
		DessinePixel();
		// Si escape est enfonc, envoyer le message WM_DESTROY dans la queue de messages
		if (Touche(VK_ESCAPE)) PostMessage(HandlePrincipale, WM_DESTROY, 0, 0);
	}
	// Nettoyer la mmoire
	CloseDirectDraw();
	// Retour  Windows
	return Msg.wParam;
}


////////////////////////////////////////////////////////////////////////////
// Remplir l'cran de 5000 pixels alatoires
void DessinePixel()
{
	// Lock la surface virtuelle afin de pouvoir crire dessus
	SurfaceSecondaire->Lock(NULL,&SurfaceDesc, DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT,NULL);
	// Ramene un ptr sur la surface secondaire
	virtuel = (UCHAR *)SurfaceDesc.lpSurface; 
	// Vide la mmoire
	memset(virtuel,0,ECRAN_X*ECRAN_Y);

	// Affiche 10000 pixels alatoires
	for (int index=0; index<10000; index++)
    {
	    int   x   = rand()%ECRAN_X;
		int   y   = rand()%ECRAN_Y;
		UCHAR col = rand()%256;
		virtuel[x+y*ECRAN_X] = col;
    } 

	// Unlock la surface virtuelle
	SurfaceSecondaire->Unlock(virtuel);	
	// copie sur la surface primaire
	while(SurfacePrimaire->Flip(NULL,DDFLIP_WAIT)!=DD_OK);
}


////////////////////////////////////////////////////////////////////////////
// Initialise DirectDraw (surface primaire & secondaire)
int InitDirectDraw()
{
	// Cration d'un objet DirectDraw
	if (DirectDrawCreate(NULL, &DirectDraw, NULL) != DD_OK)
		return 0;

	// Niveau de coopration (FullScreen et Exclusive)
	if (DirectDraw->SetCooperativeLevel(HandlePrincipale,
	                                    DDSCL_ALLOWMODEX | DDSCL_FULLSCREEN |
								        DDSCL_EXCLUSIVE | DDSCL_ALLOWREBOOT) != DD_OK)
		return 0;	
	// Slection du mode vido
	if (DirectDraw->SetDisplayMode(ECRAN_X, ECRAN_Y, 8) != DD_OK)
	{
		MessageBox(HandlePrincipale,
		           "Le mode vido dsir n'est pas support par votre pilote. Assurez-vous d'avoir la dernire version du pilote DirectX de votre carte vido.",
		           "Erreur mode vido",
		           MB_OK | MB_ICONERROR);
		return 0;
	}

	// Initialise la structure de description de la surface
	ZeroMemory(&SurfaceDesc,sizeof(SurfaceDesc));
	SurfaceDesc.dwSize            = sizeof(SurfaceDesc);
	SurfaceDesc.dwFlags           = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
	SurfaceDesc.ddsCaps.dwCaps    = DDSCAPS_PRIMARYSURFACE | DDSCAPS_SYSTEMMEMORY | DDSCAPS_FLIP | DDSCAPS_COMPLEX;
	SurfaceDesc.dwBackBufferCount = 1;

	// Cration de la surface primaire
	if (DirectDraw->CreateSurface(&SurfaceDesc, &SurfacePrimaire, NULL) != DD_OK)
		return 0;

	// Cration de la surface secondaire (virtuelle)
	SurfaceCap.dwCaps = DDSCAPS_BACKBUFFER;
	SurfacePrimaire->GetAttachedSurface(&SurfaceCap, &SurfaceSecondaire);

	// Cacher le curseur de souris
	ShowCursor(FALSE);
	return 1;
}

////////////////////////////////////////////////////////////////////////////
// Libre les objets DirectDraw
void CloseDirectDraw()
{
	// Libre surface secondaire
	if (SurfaceSecondaire != NULL)
		SurfaceSecondaire->Release();

	// Libre surface primaire
	if (SurfacePrimaire != NULL)
		SurfacePrimaire->Release();
	
	// Libre objet COM DirectDraw
	if (DirectDraw != NULL)
		DirectDraw->Release();

	// Affiche le curseur de souris
	ShowCursor(TRUE);
}

////////////////////////////////////////////////////////////////////////////